﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Ex04.Menus.Interfaces
{
    public class SubMenu : MenuItem
    {
        private const int k_BackOptionNumber = 0;
        private readonly List<MenuItem> r_ItemList;

        public List<MenuItem> ItemList
        {
            get { return r_ItemList; }
        }

        public SubMenu()
        {
            r_ItemList = new List<MenuItem>();
        }

        public SubMenu(string i_Title)
        {
            r_ItemList = new List<MenuItem>();
            Title = i_Title;
        }

        public SubMenu(string i_Title, MenuItem i_MenuSubItem)
        {
            r_ItemList = new List<MenuItem>();
            Title = i_Title;
            Add(i_MenuSubItem);
        }

        public SubMenu Add(MenuItem i_MenuSubItemToAdd)
        {
            bool isFirstItemInList = ItemList.Count == 0;
            if (isFirstItemInList)
            {
                MenuItem backItem = new ActionItem();
                backItem.Title = "Back";
                backItem.ItemNumber = 0;
                ItemList.Add(backItem);
            }

            i_MenuSubItemToAdd.ItemNumber = ItemList.Count;

            ItemList.Add(i_MenuSubItemToAdd);

            return this;
        }

        internal override void DoWhenSelected()
        {
            if (ItemList.Count == 0)
            {
                Console.WriteLine("Item not initalized yet.");
            }
            else
            {
                showSubMenu();
            }
        }

        private void showSubMenu()
        {
            int choice;

            do
            {
                Console.WriteLine(Title.ToString());
                printItemList();
                choice = getChoiceFromUser();
                Console.Clear();
                if (choice != k_BackOptionNumber)
                {
                    ItemList[choice].DoWhenSelected();
                }
            } 
            while (choice != k_BackOptionNumber);
        }

        private void printItemList()
        {
            List<MenuItem> listToPrint = new List<MenuItem>();
            string format = "{0}. {1}";

            foreach (MenuItem item in ItemList)
            {
                if (!(item.ItemNumber == k_BackOptionNumber))
                {
                    listToPrint.Add(item);
                }
            }

            foreach (MenuItem item in listToPrint)
            {
                Console.WriteLine(format, item.ItemNumber.ToString(), item.Title.ToString());
            }

            Console.WriteLine(format, ItemList[0].ItemNumber.ToString(), ItemList[0].Title.ToString());
        }

        private int getChoiceFromUser()
        {
            int choice;
            string choiceStr;

            Console.Write("Option #: ");
            choiceStr = Console.ReadLine();

            while (!int.TryParse(choiceStr, out choice) || (choice < 0 || choice > ItemList.Count - 1))
            {
                Console.Write("Please enter a number between {0} and {1}:", 0, ItemList.Count - 1);
                choiceStr = Console.ReadLine();
            }

            return choice;
        }
    }
}
